# Generated by roxygen2: do not edit by hand

export(ddbh3_bigint_to_strings)
export(ddbh3_create_conn)
export(ddbh3_default_conn)
export(ddbh3_get_center_child)
export(ddbh3_get_child_pos)
export(ddbh3_get_children)
export(ddbh3_get_icosahedron_faces)
export(ddbh3_get_n_children)
export(ddbh3_get_parent)
export(ddbh3_get_resolution)
export(ddbh3_h3_to_lat)
export(ddbh3_h3_to_lon)
export(ddbh3_h3_to_points)
export(ddbh3_h3_to_spatial)
export(ddbh3_h3_to_vertex)
export(ddbh3_h3_to_vertexes)
export(ddbh3_is_h3)
export(ddbh3_is_pentagon)
export(ddbh3_is_res_class_iii)
export(ddbh3_is_vertex)
export(ddbh3_lonlat_to_h3)
export(ddbh3_lonlat_to_spatial)
export(ddbh3_points_to_h3)
export(ddbh3_points_to_spatial)
export(ddbh3_strings_to_bigint)
export(ddbh3_vertex_to_lat)
export(ddbh3_vertex_to_lon)
export(ddbh3_vertex_to_spatial)
