#' @param name A character string of length one specifying the name of the table,
#'        or a character string of length two specifying the schema and table
#'        names. If `NULL` (the default), the function returns the result as an
#'        \code{sf} object
