% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/h3_hierarchy.R
\name{ddbh3_get_child_pos}
\alias{ddbh3_get_child_pos}
\title{Get the position of an H3 cell within its parent}
\usage{
ddbh3_get_child_pos(
  x,
  resolution = 8,
  h3 = "h3string",
  new_column = "h3child_pos",
  conn = NULL,
  name = NULL,
  overwrite = FALSE,
  quiet = FALSE
)
}
\arguments{
\item{x}{Input data. One of:
\describe{
\item{\code{duckspatial_df}}{A lazy spatial data frame via dbplyr.}
\item{\code{sf}}{A spatial data frame.}
\item{\code{tbl_lazy}}{A lazy data frame from dbplyr.}
\item{\code{data.frame}}{A standard R data frame.}
\item{character string}{A table or view name in \code{conn}.}
\item{character vector}{A vector of values to operate on in vectorized mode (requires \code{conn = NULL}).}
}}

\item{resolution}{A number specifying the resolution level of the H3
string (between 0 and 15)}

\item{h3}{The name of a column in \code{x} containing the H3 strings or
H3 unsigned 64-bit integers (\code{UBIGINT})}

\item{new_column}{Name of the new column to create on the input data. If NULL,
the function will return a vector with the result}

\item{conn}{A connection object to a DuckDB database. If \code{NULL}, the function
runs on a temporary DuckDB database.}

\item{name}{A character string of length one specifying the name of the table,
or a character string of length two specifying the schema and table
names. If \code{NULL} (the default), the function returns the result as an
\code{sf} object}

\item{overwrite}{Boolean. whether to overwrite the existing table if it exists. Defaults
to \code{FALSE}. This argument is ignored when \code{name} is \code{NULL}.}

\item{quiet}{A logical value. If \code{TRUE}, suppresses any informational messages.
Defaults to \code{FALSE}.}
}
\value{
One of the following, depending on the inputs:
\describe{
\item{\code{tbl_lazy}}{If \code{x} is not spatial.}
\item{\code{duckspatial_df}}{If \code{x} is spatial (e.g. an \code{sf} or \code{duckspatial_df} object).}
\item{\code{TRUE} (invisibly)}{If \code{name} is provided, a table is created in the connection
and \code{TRUE} is returned invisibly.}
\item{vector}{If \code{x} is a character vector and \code{conn = NULL}, the function operates
in vectorized mode, returning a vector of the same length as \code{x}.}
}
}
\description{
Get the position of H3 cell indexes stored as strings or unsigned 64-bit
integers (\code{UBIGINT}) within their parent cell at a specified resolution.
The position is a zero-based index among all children of the parent cell.
}
\examples{
## Load needed packages
library(duckh3)
library(dplyr)

## Setup the default connection with h3 and spatial extensions
## This is a mandatory step to use duckh3 functions
ddbh3_default_conn(threads = 1)

## Load example data
points_tbl <- read.csv(
  system.file("extdata/example_pts.csv", package = "duckh3")
)

## Add H3 string column
points_tbl <- ddbh3_lonlat_to_h3(points_tbl, resolution = 6)

## Get position relative to resolution 4
ddbh3_get_child_pos(points_tbl, resolution = 4)
}
